/*
   signal.c -- PIKT piktc & piktc_svc signal handling routines

   $Header: /var/cvs/PIKT/pikt/src/piktc/signal.c,v 1.33 2005/01/04 13:48:31 berto Exp $

   This file is part of PIKT ("Problem Informant/Killer Tool").
   Copyright (C) 1998,1999,2000,2001,2002,2003,2004,2005 by Robert Osterlund.

   PIKT is a trademark of the University of Chicago.  All Rights Reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   PIKT is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
   USA.

   Send problem reports, bouquets and brickbats to:  pikt-bugs@pikt.org
*/

#define SIGNAL
#include "../lib/common.h"
#include "piktc.h"
#undef SIGNAL

#include <rpc/rpc.h>
#include "rpc.h"

#ifdef HAVE_BROKEN_SA_NOCLDWAIT
# undef SA_NOCLDWAIT
#endif

struct sigaction sigchld_act;
struct sigaction sigterm_act;

#ifndef SA_NOCLDWAIT
void sig_chld(int);
#endif
void sig_term(int);

/* initialize the signal handlers
 */
void
sig_init (void)
{
	static char fnnam[] = "sig_init";

#ifdef SA_NOCLDWAIT
	sigchld_act.sa_handler = SIG_IGN;
	sigchld_act.sa_flags = SA_NOCLDWAIT;
#else
	sigchld_act.sa_handler = sig_chld;
	sigchld_act.sa_flags = 0;
#endif

	sigterm_act.sa_handler = sig_term;

	sigemptyset(&sigchld_act.sa_mask);
	if (sigaction(SIGCHLD, &sigchld_act, (struct sigaction *) NULL) < 0) {
		logexit(PIKTLOG, LOG_ERR, ERRID_SIGACTION_ERROR, __FILE__, __LINE__, fnnam, TRUE, "sigaction() error for SIGCHLD");
	}

	sigemptyset(&sigterm_act.sa_mask);
	if (sigaction(SIGTERM, &sigterm_act, (struct sigaction *) NULL) < 0) {
		logexit(PIKTLOG, LOG_ERR, ERRID_SIGACTION_ERROR, __FILE__, __LINE__, fnnam, TRUE, "sigaction() error for SIGTERM");
	}
	if (sigaction(SIGINT, &sigterm_act, (struct sigaction *) NULL) < 0) {
		logexit(PIKTLOG, LOG_ERR, ERRID_SIGACTION_ERROR, __FILE__, __LINE__, fnnam, TRUE, "sigaction() error for SIGINT");
	}
	if (sigaction(SIGQUIT, &sigterm_act, (struct sigaction *) NULL) < 0) {
		logexit(PIKTLOG, LOG_ERR, ERRID_SIGACTION_ERROR, __FILE__, __LINE__, fnnam, TRUE, "sigaction() error for SIGQUIT");
	}
}

#ifndef SA_NOCLDWAIT
/* SIGCHLD handler
 */
void
sig_chld (int sig)
{

	waitpid(-1, NULL, WNOHANG);
	return;
}
#endif

/* SIGTERM handler--on program termination, remove the piktc.lock file,
   also any files in the diffing & staging dirs; close also any open
   #include files
 */
void
sig_term (int sig)
{
	static char fnnam[] = "sig_term";
	#ifndef PIKTC_SVC
	char cmd[MAXCMD+1];
	#endif

	#ifdef PIKTC_SVC
	svc_unregister(PIKTCPROG, PIKTCVERS);
	if (rmlocks(FALSE, TRUE) == ERROR)
		logreturn(PIKTLOG, LOG_ERR, ERRID_FILE_REMOVE_ERROR, __FILE__, __LINE__, fnnam, FALSE, "rmlocks() error");
	#else
	if (rmlocks(TRUE, TRUE) == ERROR)
		logreturn(PIKTLOG, LOG_ERR, ERRID_FILE_REMOVE_ERROR, __FILE__, __LINE__, fnnam, FALSE, "rmlocks() error");
	if (isdiff) {
		sprintf(cmd, "%s %s/* 2>/dev/null", rmcmd, difdir);
		system(cmd);
	}
	sprintf(cmd, "%s %s/* 2>/dev/null", rmcmd, stgdir);
	system(cmd);
	while (incnum > 0) {
		if (cfgcurr == SYSCFG) {
			endsysincfile();
		} else {
			endtagincfile();
		}
	}
	kill(0, SIGKILL);	/* kill piktc and any of its children */
	#endif
	exit(exstat);
}
